import {
  select,
  csv,
  scaleLinear,
  scaleLog,
  extent,
  axisLeft,
  axisBottom,
  format,
} from 'd3';

const titleText = 'Daily rtb in 2015';
const xAxisLabelText = 'doy';
const yAxisLabelText = 'rtb';
const circleRadius = 5;

// set the dimensions and margins of the graph
const width = 1000;
const height = 500;

const leftWidth = width * 0.5;
const rightWidth = width - leftWidth;

const margin = 100;

// Let's create two svg elements and append them to my top-level div
// Here's the first svg, and I'll save the selection to the variable "svg"
const svg = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', leftWidth)
  .attr('height', height);

// And the second; the ".append" method returns a selection with just
// the svg in it, so I can manipulate it later (by appending or joining
// data with it)
const svg2 = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', rightWidth)
  .attr('height', height);

const tooltip = d3
  .select('#my_dataviz')
  .append('div')
  .attr('id', 'tooltip');

// render
const render = (rtbData, tempData) => {
  const xValue = (d) => d.doy;
  const yValue = (d) => d.rtb;
  const margin = {
    top: 77,
    right: 500,
    bottom: 99,
    left: 10,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const xScale = scaleLinear()
    // .domain(extent(rtbData, xValue))
    .domain([0, 370])
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(rtbData, yValue))
    .range([innerHeight, 0])
    .nice();

  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);

  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(15);

  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();

  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -125)
    .attr('x', -innerHeight / 2)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabelText);

  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG.select('.domain').remove();

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 65)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabelText);

  g.append('text')
    .attr('class', 'title')
    .attr('y', -25)
    .text(titleText);

  const marks = g
    .selectAll('circle')
    .data(rtbData)
    .enter()
    .append('circle')
    .attr('cy', (d) => yScale(yValue(d)))
    .attr('cx', (d) => xScale(xValue(d)))
    .attr('r', circleRadius)
    .attr('id', (d) => d.id)
    .style('fill', 'steelblue')
    .style('opacity', 0.5);

  function highlightMarks() {
    d3.select(this)
      .attr('r', circleRadius * 2)
      .style('fill', 'red')
      .style('opacity', 1);
  }

  function resetMarks() {
    d3.select(this)
      .transition()
      .duration(100)
      .attr('r', circleRadius)
      // hardcoded eek, maybe css is dif with interactivity
      .attr('fill', 'steelblue')
      .style('opacity', 0.5);
  }

  function popUpTooltip(event, data) {
    // let country = countryData.get(data.properties.name);
    tooltip.style('opacity', 1);
    tooltip.html(
      'doy: ' + data.doy + '<br>rtb: ' + data.rtb,
    );
  }

  function adjustTooltip(event) {
    tooltip
      .style('left', event.x - 80 + 'px')
      .style('top', event.y - 80 + 'px');
  }

  marks
    // idk why the event logs correctly here
    .on('mouseover', function (event, data) {
      //highlightMarks();
      d3.select(this)
        .attr('r', circleRadius * 2)
        .style('fill', 'red')
        .style('opacity', 1);
      popUpTooltip(event, data);
      console.log(event);
      console.log(data);
      setDoy(+this.id + 1);
      updateSvg2();
    })
    .on('mousemove', function (event, data) {
      adjustTooltip(event);
    })
    .on('mouseleave', function (event, data) {
      //resetMarks();
      d3.select(this)
        .transition()
        .duration(600)
        .attr('r', circleRadius)
        // hardcoded eek, maybe css is dif with interactivity
        .style('fill', 'steelblue')
        .style('opacity', 0.5);
      d3.select('#tooltip').style('opacity', 0);
    });

  // START ON LINEPLOT HERE
  // BASICALLY, save doy on left in var and siplay that
  // index of tempData here

  // Declare the chart dimensions and margins.
  const width2 = 750;
  const height2 = 500;
  const marginTop = 100;
  const marginRight = 30;
  const marginBottom = 100;
  const marginLeft = 40;

  // Declare the x (horizontal position) scale.
  const x = d3
    .scaleLinear()
    .domain(d3.extent(tempData, (d) => d.hour))
    .range([marginLeft, width2 - marginRight])
    .nice();

  // Declare the y (vertical position) scale.
  const y = d3
    .scaleLinear()
    .domain(d3.extent(tempData, (d) => d.temp_F).reverse())
    // .domain([0, 50].reverse())
    .range([marginTop, height2 - marginBottom])
    .nice();

  // Declare the line generator.
  const line = d3
    .line()
    .x((d) => x(d.hour))
    .y((d) => y(d.temp_F));

  // Create the SVG container.
  // const svg2 = d3
  //   .select('svg')
  //   .attr('width', width2)
  //   .attr('height', height2)
  //   .attr('viewBox', [0, 0, width2, height2])
  //   .attr(
  //     'style',
  //     'max-width: 100%; height: auto; height: intrinsic;',
  //   );

  // Add the x-axis.
  svg2
    .append('g')
    .attr(
      'transform',
      `translate(0,${height2 - marginBottom})`,
    )
    .call(
      d3
        .axisBottom(x)
        .ticks(width / 80)
        .tickSizeOuter(0),
    );

  // Add the y-axis, remove the domain line, add grid lines and a label.
  svg2
    .append('g')
    .attr('transform', `translate(${marginLeft},0)`)
    .call(d3.axisLeft(y).ticks(height / 40))
    .call((g) => g.select('.domain').remove())
    .call((g) =>
      g
        .selectAll('.tick line')
        .clone()
        .attr('x2', width2 - marginLeft - marginRight)
        .attr('stroke-opacity', 0.1),
    )
    .call((g) =>
      g
        .append('text')
        .attr('class', 'label')
        .attr('x', -marginLeft)
        .attr('y', 10)
        .attr('fill', 'currentColor')
        .attr('text-anchor', 'start')
        .text('doy: ' + getDoy()),
    );

  const updateSvg2 = () => {
    svg2
      .append('g')
      .attr('transform', `translate(${marginLeft},0)`)
      .call(d3.axisLeft(y).ticks(height / 40))
      .call((g) =>
        g
          .append('text')
          .attr('x', -marginLeft)
          .attr('y', 10)
          .attr('fill', 'currentColor')
          .attr('text-anchor', 'start')
          .text('doy: ' + getDoy()),
      );
    tempsUrl = getTempsUrl(getDoy());
    Promise.all([d3.csv(rtbUrl), d3.csv(tempsUrl)]).then(
      ([rtbData, tempData]) => {
        rtbData.forEach((row) => {
          row.year = +row.year;
          row.doy = +row.doy;
          row.rtb = +row.rtb;
        });
        tempData.forEach((row) => {
          row.hour = +row.hour;
          row.temp_F = +row.temp_F;
        });
        rtbData = rtbData.filter(getRtbYear);
        console.log(rtbData);
        console.log(tempData);
        render(rtbData, tempData);
      },
    );
  };

  // // Append a path for the line.
  // svg2
  //   .append('path')
  //   .attr('fill', 'none')
  //   .attr('stroke', 'steelblue')
  //   .attr('stroke-width', 1.5)
  //   .attr('d', line(tempData));
};

const year = 2015;
var doy = 1;

const getDoy = () => {
  return doy;
};
const setDoy = (d) => {
  doy = d;
};

function getTempsUrl(doy) {
  return (
    'https://raw.githubusercontent.com/obiekahne/cp341/main/temps_doy' +
    doy +
    '.csv'
  );
}

// read the data
const rtbUrl =
  'https://raw.githubusercontent.com/obiekahne/cp341/main/rtb.csv';
var tempsUrl = getTempsUrl(doy);

function getRtbYear(row) {
  return row['year'] == year;
}

Promise.all([d3.csv(rtbUrl), d3.csv(tempsUrl)]).then(
  ([rtbData, tempData]) => {
    rtbData.forEach((row) => {
      row.year = +row.year;
      row.doy = +row.doy;
      row.rtb = +row.rtb;
    });
    tempData.forEach((row) => {
      row.hour = +row.hour;
      row.temp_F = +row.temp_F;
    });
    rtbData = rtbData.filter(getRtbYear);
    console.log(rtbData);
    console.log(tempData);
    render(rtbData, tempData);
  },
);
